
MERGE_MIN_CONFIDENCE = 0.6
PARTIAL_MIN_CONFIDENCE = 0.1
THRESHOLD_INBETWEEN_MERGE = 0.7
TEXT_DISTANCE_MERGE_THRESHOLD = 1000
TEXT_DISTANCE_CLOSE_STATEMENT_DETECTION = TEXT_DISTANCE_MERGE_THRESHOLD
MAX_DISTANCE_UNIQUE_PROXIMITY = 20000
ELEMENTS_WORDS_TO_INCLUDE = 50
ELEMENTS_TABLES_TO_INCLUDE = 3
NUM_TOKENS_DEFAULT_LLM = 4000

ID_NOT_AVAILABLE = "[NOT_SPECIFIED]"
ID_NOT_CONFIGURED = "[NOT_CONFIGURED]"
